@@ -49,6 +49,7 @@ def get_cluster_target_status(cluster_id):
         else:
             offline_nodes += 1
 
+
     logger.debug(f"online_nodes: {online_nodes}")
     logger.debug(f"offline_nodes: {offline_nodes}")
     logger.debug(f"affected_nodes: {affected_nodes}")
@@ -70,48 +71,130 @@ def get_cluster_target_status(cluster_id):
     return Cluster.STATUS_ACTIVE
 
 
+def is_new_migrated_node(cluster_id, node):
+    for dev in node.nvme_devices:
+        if dev.status == NVMeDevice.STATUS_ONLINE:
+            for item in node.lvstore_stack:
+                if item["type"] == "bdev_distr":
+                    if tasks_controller.get_new_device_mig_task(cluster_id, node.uuid, item["name"],dev.get_id()):
+                        return True
+    return False
+
+
+def get_current_cluster_status(cluster_id):
+    cluster = db_controller.get_cluster_by_id(cluster_id)
+    if cluster.status == cluster.STATUS_UNREADY:
+        return Cluster.STATUS_UNREADY
+    snodes = db_controller.get_primary_storage_nodes_by_cluster_id(cluster_id)
+
+    online_nodes = 0
+    offline_nodes = 0
+    affected_nodes = 0
+    online_devices = 0
+    offline_devices = 0
+
+    for node in snodes:
+
+        node_online_devices = 0
+        node_offline_devices = 0
+
+        if node.status == StorageNode.STATUS_IN_CREATION:
+            continue
+
+        if node.status == StorageNode.STATUS_ONLINE:
+            if is_new_migrated_node(cluster_id, node):
+                continue
+            online_nodes += 1
+        elif node.status == StorageNode.STATUS_REMOVED:
+            pass
+        else:
+            offline_nodes += 1
+        for dev in node.nvme_devices:
+            if dev.status in [NVMeDevice.STATUS_ONLINE, NVMeDevice.STATUS_JM, NVMeDevice.STATUS_READONLY]:
+                node_online_devices += 1
+            elif dev.status == NVMeDevice.STATUS_FAILED_AND_MIGRATED:
+                pass
+            else:
+                node_offline_devices += 1
+
+        if node_offline_devices > 0 or node_online_devices == 0:
+            affected_nodes += 1
+
+        online_devices += node_online_devices
+        offline_devices += node_offline_devices
+
+
+    logger.debug(f"online_nodes: {online_nodes}")
+    logger.debug(f"offline_nodes: {offline_nodes}")
+    logger.debug(f"affected_nodes: {affected_nodes}")
+    logger.debug(f"online_devices: {online_devices}")
+    logger.debug(f"offline_devices: {offline_devices}")
+    # ndcs n = 2
+    # npcs k = 1
+    n = cluster.distr_ndcs
+    k = cluster.distr_npcs
+
+    # if number of devices in the cluster unavailable on DIFFERENT nodes > k --> I cannot read and in some cases cannot write (suspended)
+    if affected_nodes > k:
+        return Cluster.STATUS_SUSPENDED
+    # if number of devices in the cluster available < n + k --> I cannot write and I cannot read (suspended)
+    if online_devices < n + k:
+        return Cluster.STATUS_SUSPENDED
+    if cluster.strict_node_anti_affinity:
+        # if (number of online nodes < number of total nodes - k) --> suspended
+        if online_nodes < (len(snodes) - k):
+            return Cluster.STATUS_SUSPENDED
+        # if (number of online nodes < n+1) --> suspended
+        if online_nodes < (n + 1):
+            return Cluster.STATUS_SUSPENDED
+        # if (number of online nodes < n+2 and k=2) --> suspended
+        if online_nodes < (n + 2) and k == 2:
+            return Cluster.STATUS_SUSPENDED
+    else:
+        # if (number of online nodes < number of total nodes - k) --> degraded
+        if online_nodes < (len(snodes) - k):
+            return Cluster.STATUS_DEGRADED
+        # if (number of online nodes < n+1) --> degraded
+        if online_nodes < (n + 1):
+            return Cluster.STATUS_DEGRADED
+        # if (number of online nodes < n+2 and k=2) --> degraded
+        if online_nodes < (n + 2) and k == 2:
+            return Cluster.STATUS_DEGRADED
+
+    return Cluster.STATUS_ACTIVE
+
 def update_cluster_status(cluster_id):
     cluster = db_controller.get_cluster_by_id(cluster_id)
 
-    if cluster.ha_type == "ha":
+    if cluster.status == Cluster.STATUS_READONLY or cluster.status == Cluster.STATUS_UNREADY \
+            or Cluster.STATUS_IN_ACTIVATION:
+        return
+    cluster_current_status = get_current_cluster_status(cluster_id)
+    logger.info("cluster_status: %s", cluster.status)
+    logger.info("cluster_new_status: %s", cluster_current_status)
+    if cluster.status not in [Cluster.STATUS_ACTIVE, Cluster.STATUS_UNREADY] and cluster_current_status == Cluster.STATUS_ACTIVE:
+        # cluster_ops.cluster_activate(cluster_id, True)
+        cluster_ops.set_cluster_status(cluster_id, Cluster.STATUS_ACTIVE)
+        return
+    cluster_ops.set_cluster_status(cluster_id, cluster_current_status)
 
-        if cluster.status == Cluster.STATUS_READONLY:
-            return
-        if cluster.status == Cluster.STATUS_UNREADY:
-            return
-
-        cluster_target_status = get_cluster_target_status(cluster_id)
-        logger.info(f"Target cluster status {cluster_target_status}, current status: {cluster.status}")
-        if cluster.status == cluster_target_status:
-            return
-
-        if cluster_target_status == Cluster.STATUS_ACTIVE:
-            logger.info(f"Resuming cluster: {cluster_id}")
-            cluster_ops.unsuspend_cluster(cluster_id)
-
-        elif cluster_target_status == Cluster.STATUS_SUSPENDED:
-            logger.warning(f"Suspending cluster: {cluster_id}")
-            cluster_ops.suspend_cluster(cluster_id)
-
-        elif cluster_target_status == Cluster.STATUS_DEGRADED:
-            logger.warning(f"Degrading cluster: {cluster_id}")
-            cluster_ops.degrade_cluster(cluster_id)
 
 
 def set_node_online(node):
     if node.status != StorageNode.STATUS_ONLINE:
+
+        # set node online
+        storage_node_ops.set_node_status(node.get_id(), StorageNode.STATUS_ONLINE)
+
+        # set jm dev online
+        if node.jm_device.status in [JMDevice.STATUS_UNAVAILABLE, JMDevice.STATUS_ONLINE]:
+            device_controller.set_jm_device_state(node.jm_device.get_id(), JMDevice.STATUS_ONLINE)
+
         # set devices online
         for dev in node.nvme_devices:
             if dev.status == NVMeDevice.STATUS_UNAVAILABLE:
                 device_controller.device_set_online(dev.get_id())
 
-        # set jm dev online
-        if node.jm_device.status == JMDevice.STATUS_UNAVAILABLE:
-            device_controller.set_jm_device_state(node.jm_device.get_id(), JMDevice.STATUS_ONLINE)
-
-        # set node online
-        storage_node_ops.set_node_status(node.get_id(), StorageNode.STATUS_ONLINE)
-
 
 def set_node_offline(node):
     if node.status != StorageNode.STATUS_UNREACHABLE:
@@ -120,12 +203,12 @@ def set_node_offline(node):
 
         # set devices unavailable
         for dev in node.nvme_devices:
-            if dev.status == NVMeDevice.STATUS_ONLINE:
+            if dev.status in [NVMeDevice.STATUS_ONLINE, NVMeDevice.STATUS_READONLY]:
                 device_controller.device_set_unavailable(dev.get_id())
 
-        # set jm dev offline
-        if node.jm_device.status != JMDevice.STATUS_UNAVAILABLE:
-            device_controller.set_jm_device_state(node.jm_device.get_id(), JMDevice.STATUS_UNAVAILABLE)
+        # # set jm dev offline
+        # if node.jm_device.status != JMDevice.STATUS_UNAVAILABLE:
+        #     device_controller.set_jm_device_state(node.jm_device.get_id(), JMDevice.STATUS_UNAVAILABLE)
 
 
 logger.info("Starting node monitor")
@@ -195,7 +278,7 @@ while True:
                     # restart on new node
                     storage_node_ops.set_node_status(snode.get_id(), StorageNode.STATUS_SCHEDULABLE)
 
-                elif ping_check and node_api_check and not spdk_process:
+                elif ping_check and node_api_check and (not spdk_process or not node_rpc_check):
                     # add node to auto restart
                     if cluster.status == Cluster.STATUS_ACTIVE:
                         tasks_controller.add_node_to_auto_restart(snode)
